%%% -*- coding: latin-1 -*-

%%% This file is part of RefactorErl.
%%%
%%% RefactorErl is free software: you can redistribute it and/or modify
%%% it under the terms of the GNU Lesser General Public License as published
%%% by the Free Software Foundation, either version 3 of the License, or
%%% (at your option) any later version.
%%%
%%% RefactorErl is distributed in the hope that it will be useful,
%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%% GNU Lesser General Public License for more details.
%%%
%%% You should have received a copy of the GNU Lesser General Public License
%%% along with RefactorErl.  If not, see <http://plc.inf.elte.hu/erlang/>.
%%%
%%% The Original Code is RefactorErl.
%%%
%%% The Initial Developer of the Original Code is Eötvös Loránd University.
%%% Portions created  by Eötvös Loránd University and ELTE-Soft Ltd.
%%% are Copyright 2007-2013 Eötvös Loránd University, ELTE-Soft Ltd.
%%% and Ericsson Hungary. All Rights Reserved.

%%% @doc This module is a GUI for duplicated code analysis
%%% @author Gabor Hosszu

-module(referl_wx_dupcode).
-author("Gábor Hosszú").
-behaviour(wx_object).
 
-export([start/1, init/1, terminate/2,  code_change/3,
         handle_info/2, handle_call/3, handle_cast/2, handle_event/2]).
 
-include_lib("referl_core/include/core_export.hrl").
-include_lib("wx/include/wx.hrl").
-include("wx.hrl").
-include("ui.hrl").
 
-record(state,
{
 parent,
 main_pid,
 access_to_db,
 runb,
 group_ch,
 choice,
 codebrs,
 codechs,
 setup,      %% User input fields
 fileinp,    %% Input field for filename or regexp
 filelist,
 nodes1,     %% RefactorErl nodes of the currently selected codes (const_var_diff)
 nodes2
    }).

-define(runButton, 4010).
-define(addButton, 4011).
-define(delButton, 4012).
-define(difButton, 4013).

-define(grpChooser, 4101).
-define(code1Chooser, 4102).
-define(code2Chooser, 4103).
 
%% @private
start(Props) ->
    wx_object:start_link(?MODULE, Props, []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% @private
init(Props) ->
        wx:batch(fun() -> do_init(Props) end).
 
do_init(Props) ->
    Panel = proplists:get_value(parent, Props),
    MainPid = proplists:get_value(main_pid, Props, self()),

    %% MAC RELATED
    %% On Mac, wxStaticBoxSizers must be "defined" before the controls
    FilesSz = wxStaticBoxSizer:new(?wxVERTICAL, Panel, [{label, "Files"}]),
    SetupWSz = wxStaticBoxSizer:new(?wxVERTICAL, Panel, [{label, "Setup"}]),
    RunB = wxButton:new(Panel, ?runButton, [{label,"Run analysis"},
                                            {size, {150, 25}}]),
    DifB = wxButton:new(Panel, ?difButton, [{label,"Show diffs"},
                                            {size, {150, 25}}]),
    CodeGroupCh = wxChoice:new(Panel, ?grpChooser, [{size, {210, 25}}]),
    CH1 = wxChoice:new(Panel, ?code1Chooser, [{size, {100, 25}}]),
    CH2 = wxChoice:new(Panel, ?code2Chooser, [{size, {100, 25}}]),
    CB1 = referl_wx_codebrowser:start_browser(Panel, MainPid),
    CB2 = referl_wx_codebrowser:start_browser(Panel, MainPid),
    FilesInput = wxStyledTextCtrl:new(Panel, [{size, {210, 20}}]),
    ?wxMain:set_stc_style(FilesInput),
    %% wxTextCtrl:new(Panel, 12500, [{size, {210, 25}}]),
    FilesAdd = wxButton:new(Panel, ?addButton, [{label,"Add file/regexp"},
                                                    {size, {105, 25}}]),
    FilesDel = wxButton:new(Panel, ?delButton, [{label,"Delete item"},
                                                    {size, {105, 25}}]),
    FilesL = wxListBox:new(Panel, 4450, [{size, {210, 200}},
                                          {style, ?wxLB_EXTENDED}]),
    MinLen = wxTextCtrl:new(Panel, 4500, [{size, {40, 25}}]),
    MinNum = wxTextCtrl:new(Panel, 4501, [{size, {40, 25}}]),
    OvLap  = wxTextCtrl:new(Panel, 4502, [{size, {40, 25}}]),
    MLTxt = wxStaticText:new(Panel, 4511, "Minimum length:", []),
    MNTxt  = wxStaticText:new(Panel, 4512, "Minimum number of dups", []),
    OlTxT  = wxStaticText:new(Panel, 4513, "Overlap amount:", []),
    CB1TxT = wxStaticText:new(Panel, 4514, "Left browser and right browser", []),
    %% CB2TxT = wxStaticText:new(Panel, 4515, "Right browser", []),
    CodeGroupTxT = wxStaticText:new(Panel, 4516, "Group", []),
    wxTextCtrl:setValue(MinLen, "64"),
    wxTextCtrl:setValue(MinNum, "2"),
    wxTextCtrl:setValue(OvLap, "0"),

    Expand  = [{flag, ?wxEXPAND}, {proportion, 1}],

    MainSizer = wxBoxSizer:new(?wxHORIZONTAL),
        
    SetPaneSz = wxBoxSizer:new(?wxVERTICAL),
    BtnSz = wxBoxSizer:new(?wxHORIZONTAL),
    CodeCH12Sz = wxBoxSizer:new(?wxHORIZONTAL),
    % CodeCHTxt12Sz = wxBoxSizer:new(?wxHORIZONTAL),
    TextCtrlSz = wxBoxSizer:new(?wxHORIZONTAL),
    
    BtnFlags = [{border, 6}, {flag, ?wxCENTRE bor ?wxALL}],
    wxSizer:add(FilesSz, FilesInput),
    wxSizer:addSpacer(FilesSz, 10),
    wxSizer:add(BtnSz, FilesAdd),
    wxSizer:add(BtnSz, FilesDel),
    wxSizer:add(FilesSz, BtnSz),
    wxSizer:addSpacer(FilesSz, 10),
    wxSizer:add(FilesSz, FilesL),
    wxSizer:add(SetPaneSz, FilesSz),
    wxSizer:addSpacer(SetPaneSz, 5),
    wxSizer:add(SetupWSz, MLTxt),
    wxSizer:add(SetupWSz, MinLen),
    wxSizer:add(SetupWSz, MNTxt),
    wxSizer:add(SetupWSz, MinNum),
    wxSizer:add(SetupWSz, OlTxT),
    wxSizer:add(SetupWSz, OvLap),
    wxSizer:add(SetPaneSz, SetupWSz, [{flag, ?wxEXPAND}]),
    wxSizer:addSpacer(SetPaneSz, 10),
    wxSizer:add(SetPaneSz, RunB, BtnFlags), 
    wxSizer:add(SetPaneSz, DifB, BtnFlags), 
    wxSizer:add(SetPaneSz, CodeGroupTxT, BtnFlags),
    wxSizer:add(SetPaneSz, CodeGroupCh, BtnFlags),
    wxSizer:add(CodeCH12Sz, CH1, BtnFlags),
    wxSizer:add(CodeCH12Sz, CH2, BtnFlags),
    wxSizer:add(SetPaneSz, CB1TxT, BtnFlags),
    wxSizer:add(SetPaneSz, CodeCH12Sz),
    wxSizer:add(SetPaneSz, ?wxMain:make_placeholder(Panel, 200, 1), []),
    wxSizer:add(TextCtrlSz, CB1, Expand),
    wxSizer:add(TextCtrlSz, CB2, Expand),
    wxSizer:add(MainSizer, SetPaneSz, 
                           [{border, 10}, {flag, ?wxEXPAND}, {proportion, 0}]),
    wxSizer:add(MainSizer, TextCtrlSz, 
                           [{border, 10}, {flag, ?wxEXPAND}, {proportion, 1}]),
    

    ?wxMain:enable_btns([CH1, CH2, CodeGroupCh], false),

    %% CONNECT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    wxWindow:connect(FilesInput, stc_charadded), 
    wxWindow:connect(FilesInput, stc_userlistselection),
    wxWindow:connect(Panel, command_button_clicked),
    wxWindow:connect(Panel, command_choice_selected),
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %% MAC RELATED
    wxPanel:setSizer(Panel, MainSizer),
    wxWindow:fit(Panel),
    wxSizer:fit(SetPaneSz, Panel),
    wxSizer:layout(MainSizer),

    {Panel, #state{parent=Panel, runb=RunB, group_ch=CodeGroupCh, choice=none,
                   codebrs={CB1, CB2}, codechs={CH1, CH2}, filelist=FilesL,
                   fileinp=FilesInput, setup={MinLen, MinNum, OvLap},
                   access_to_db=true, main_pid=MainPid}}.
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% @private
%% Choosing code group
handle_event(#wx{id=?grpChooser,event=#wxCommand{type=command_choice_selected} }, 
         State = #state{ group_ch=CodeGroupCh, choice=Ch, codechs=CCs}) ->
        WxCh = wxChoice:getSelection(CodeGroupCh),
        if Ch =/= WxCh ->
                   Group = wxChoice:getClientData(CodeGroupCh, WxCh),
                   handle_dup_group_load(Group, CCs);

           true -> ok
        end,
    {noreply,State#state{ choice=WxCh }};

%% Choosing code 1 and code 2 
handle_event(#wx{id=?code1Chooser,
         event=#wxCommand{ type=command_choice_selected } }, 
         State = #state{ codebrs={CB1, _CB2}, codechs={CH1, _CH2}}) ->
        Nodes = load_code(CB1,CH1),
    {noreply,State#state{ nodes1 = Nodes }};
handle_event(#wx{id=?code2Chooser,
         event=#wxCommand{ type=command_choice_selected } }, 
         State = #state{ codebrs={_CB1, CB2}, codechs={_CH1, CH2}}) ->
        Nodes = load_code(CB2,CH2),
    {noreply,State#state{ nodes2 = Nodes }};

%% Button clicks
handle_event(#wx{id=Id, event=#wxCommand{ type=command_button_clicked } }, 
      State = #state{ runb=RunB, group_ch=CodeGroupCh, codechs=CHs={CH1, CH2},
                      setup=Input, fileinp=FInput, filelist=FList, choice=Ch,
                      nodes1=Nodes1, nodes2=Nodes2, codebrs={CB1, CB2} } ) ->
    Ch2 =
	case Id of
		?runButton    -> Btns = [RunB, CodeGroupCh, CH1, CH2],
                         run_dupcode_anal(Btns, FList, CodeGroupCh, Input, CHs,
                                          State#state.main_pid),
                         none;
        ?addButton    -> add_file_or_regexp(FInput,FList),
                         Ch;
        ?delButton    -> del_file_or_regexp(FList),
                         Ch;
        ?difButton    -> show_diff(CB1, CB2, Nodes1, Nodes2),
                         Ch;
        _             -> Ch
	end,
    {noreply,State#state{ choice=Ch2 }};

%% Autocomplete for starting node input
handle_event(#wx{event = #wxStyledText{type = stc_charadded}, obj=STC }, State) ->
    Txt = wxStyledTextCtrl:getText(STC),
    Res = ?HandleRes(?ExecLCall( autocomplete_funmod, [Txt, modpath]), [] ),
    wxStyledTextCtrl:autoCompSetMaxWidth(STC, 6000),
    CompleteList = string:join(Res, ?SEP),
    wxStyledTextCtrl:userListShow(STC, 1, CompleteList),
    {noreply,State};

handle_event(#wx{event=S=#wxStyledText{type = stc_userlistselection}, obj=STC }, State) ->
    wxStyledTextCtrl:setText(STC, S#wxStyledText.text),
    {noreply,State};

handle_event(#wx{}, State = #state{}) ->
    {noreply, State}.
    
%% @private
handle_cast(_Msg, State) ->
    {noreply,State}.

%% @private
%% DB access
handle_info({event, db_access, Bool},State=#state{ runb=RunB }) ->
    ?wxMain:enable_btns([RunB], Bool),
    {Br1, Br2} = State#state.codebrs,
    Brws = [Br1, Br2],
    send_browser_event(Brws,db_access,Bool),
    {noreply, State#state{ access_to_db=Bool }};

handle_info({event, Evt, Info}, State) when Info =/= none ->
    {Br1, Br2} = State#state.codebrs,
    Brws = [Br1, Br2],
    send_browser_event(Brws,Evt,Info),
    {noreply, State};
     
%% Callbacks handled as normal gen_server callbacks
handle_info(_Msg, State) ->
    {noreply, State}.
 
%% @private
handle_call(_Msg, _From, State) ->
    {reply, {error, nyi}, State}.
 
%% @private
code_change(_, _, State) ->  %%this is just a stub
    {stop, ignore, State}.
 
%% @private
terminate(_Reason, _State) ->
    ok.
    
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Local functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Run duplicated code analisys
%% L1 and L2 are the two dup code choice lists
run_dupcode_anal(Btns, FileList, GroupChooser, Input, {L1, L2}, Main) -> 
    {MinLen, MinNum, OvLap} = Input,
    MLVal = list_to_int(wxTextCtrl:getValue(MinLen), 64),
    MNVal = list_to_int(wxTextCtrl:getValue(MinNum),  2),
    OvVal = list_to_int(wxTextCtrl:getValue(OvLap),   0),
    Files = [ wxListBox:getString(FileList, I) || 
                           I <- lists:seq(0,wxListBox:getCount(FileList) - 1) ],
    ?wxMain:enable_btns(Btns, false),
    ?wxMain:statusbar_notification(Main, "Duplicated code analysis started.", 
                                                    ?stbar_Notif_Timeout),
    Params = [{files, Files}, {minlen, MLVal}, {minnum, MNVal}, 
              {overlap, OvVal}, {postype, scalar}],
    ?HandleRes( ?ExecLCall(search_initial_clones, [Params]), 
                         fun handle_dup_result/2, {GroupChooser, {L1, L2}} ),
    ?wxMain:statusbar_notification(Main, "Duplicated code analysis finished.", 
                                                    ?stbar_Notif_Timeout),
    ?wxMain:infowindow_main("Duplicated code analysis finished", "Info"),
    ?wxMain:enable_btns(Btns, true).

%% Handler for duplicated code analisys results
handle_dup_result(Res, {GroupChooser, {L1, L2}}) ->
    fill_choice_list(GroupChooser, Res, "Duplicate group"),
    handle_dup_group_load([], {L1, L2}).

%% Proplist of a code ("dup")
%% Fill L1 and L2 code choice lists
handle_dup_group_load(Group, {L1, L2}) ->
    fill_code_choice_list(L1, Group),
    fill_code_choice_list(L2, Group).


fill_code_choice_list(CodeChooser, Group) -> 
    fill_choice_list(CodeChooser, Group, "Code").

fill_choice_list(WxChoice, Items, Title) ->    
    wxChoice:clear(WxChoice),
    NumNRes = lists:zip(lists:seq(1,length(Items)), Items),
    FunToRun =
    fun({Index, Item}) ->
        Text = Title ++ " " ++ integer_to_list(Index),
        wxChoice:append(WxChoice, Text, Item)
    end,
    wx:foreach(FunToRun, NumNRes).   

load_code(CB,CH) ->
    Chosen = wxChoice:getSelection(CH),
    Data = wxChoice:getClientData(CH, Chosen),
    File = proplists:get_value(filepath, Data),
    St   = proplists:get_value(startpos, Data),
    End  = proplists:get_value(endpos, Data),
    Props = [{file, File}, {pos, {St, End}}],
    referl_wx_codebrowser:load_code(CB, Props),
    proplists:get_value(nodes, Data, []).

add_file_or_regexp(FInput,FList) ->
    case wxStyledTextCtrl:getText(FInput) of
            ""  -> ok;
            F   -> wxListBox:append(FList, F)
    end.

del_file_or_regexp(FList) ->
    case wxListBox:getSelections(FList) of
        {0, _} -> ok;
        {_, S} -> [ wxListBox:delete(FList, I) || I <- lists:reverse(S) ]
    end.

list_to_int(L, Default) ->
    try
        list_to_integer(L)
    catch
        _:_ -> Default
    end.

show_diff(_, _, undefined, undefined) -> ok;
show_diff(Browser1, Browser2, Nodes1, Nodes2) ->
    R = ?ExecLCall( get_const_var_diff, [Nodes1, Nodes2] ),
    case R of
        {[], []}     -> ?wxMain:infowindow_main("There are no differences.",
                                                "No differences"),
                        ok;
        {Pos1, Pos2} -> ScPos1 = [ Sc || {Sc, _} <- Pos1 ],
                        ScPos2 = [ Sc || {Sc, _} <- Pos2 ],
                        ?wxCB:highlight_positions(Browser1, ScPos1),
                        ?wxCB:highlight_positions(Browser2, ScPos2);
        _            -> error
    end.

%% Send an event to the code browsers started from this module
send_browser_event(Browsers, Evt, Info) ->
    [ wx_object:get_pid(Br) ! {browser_event, Evt, Info} 
                                                    || Br <- Browsers ].