'use strict';

function DuplicatesCtrl($scope, autoComplete, duplicates, $http, locationSearch, $location) {

	duplicates.setupNavigationIn($scope);

	$scope.historyType = "duplicates";

	// used by the view to extend options
	$scope.extend = angular.extend;

	// The main model
	$scope.parameters = {
		subjects: [],
		minLength: "64",
		minNum: "2",
		overlap: "0"
	};

	$scope.subjectsConfig = {
		multiple: true,
		minimumInputLength: 1,
		query: autoComplete.createFullModuleQuery()
	};

	// Parameters binding
	$scope.$watch(duplicates.parameters, function(parameters, old) {
		if (old === parameters) return; // don't clear with the initial empty parameters
		$scope.parameters = parameters;
	});

	// Results
	$scope.waiting = duplicates.waiting;
	$scope.results = duplicates.lastResults;
	$scope.idHref = locationSearch.getterForPart("id");

	$scope.generate = function() {
		duplicates.execute($scope.parameters);
	};

	$scope.codeMirrorConfig = {
		lineNumbers: true,
		mode: 'erlang',
		readOnly: true,
		styleSelectedText: true
	};
	$scope.panels = [
		{ path: "path1", content: "" },
		{ path: "path2", content: "" }
	];
	$scope.itemsConfig = {
		query: function(query) {
			var group = $scope.currentGroup;
			query.callback({results: group ? group.items : []});
		}
	};

	var update = function() {
		if ($location.path() != "/duplicates") return;
		var group = locationSearch.get("group");
		if (!group) {
			group = 0;
			locationSearch.set("group", group);
			$location.replace();
		}
		var items = locationSearch.get("items");
		if (!items) {
			items = "0,1";
			locationSearch.set("items", items);
			$location.replace();
		}
		var results = duplicates.lastResults();
		if (results) {
			var currentGroup = results.groups[group];
			$scope.currentGroup = currentGroup;
			if (currentGroup) {
				angular.forEach(currentGroup.items, function(item, index) {
					item.id = index;
					var fileName = item.path.replace(/.*\//, "");
					item.text = fileName + ":" + item.start.line;
				});
				var indexes = items.split(",");
				angular.forEach(indexes, function(itemIndex, panelIndex) {
					$scope.panels[panelIndex].item = currentGroup.items[itemIndex];
				});
			}
		}
	};

	var getItems = function() {
		return _.map($scope.panels, function(panel) {
			return panel.item || $scope.currentGroup.items[0];
		});
	};

	var getItemIndexes = function() {
		return _.map(getItems(), function(item) { return item.id; }).join(",");
	};

	// for synchronized scroll position
	var suffix =
		"\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n" +
		"\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n";

	angular.forEach($scope.panels, function(panel, panelIndex) {
		var item = function() { return panel.item };
		$scope.$watch(item, function(item) {
			if (item) {
				locationSearch.set("items", getItemIndexes());
				item.highlight = [item.start, item.end];
				angular.forEach($scope.currentGroup.items, function(item) {
					item.multiHighlight = [];
				});
				$http.get("api/getFile", {params: {file: item.path}}).success(function(file) {
					item.content = file.content + suffix;
				});
			}
		});
	});

	$scope.showDiff = function() {
		var items = getItems();
		var url = "api/duplicates/diff/" + duplicates.foregroundId() + "/" + locationSearch.get("group");
		$http.get(url, {params: {items: getItemIndexes()}}).success(function(response) {
			angular.forEach(response.ranges, function(range, index) {
				items[index].multiHighlight = range;
			});
		});
	};

	$scope.$on("$routeUpdate", update);

	$scope.$watch(duplicates.lastResults, update);

}
