'use strict';

function GraphsCtrl($scope, autoComplete, graphs, locationSearch, $http, $q) {

	graphs.setupNavigationIn($scope);

	$scope.historyType = "graphs";

	// used by the view to extend options
	$scope.extend = angular.extend;

	// The main model
	$scope.parameters = {
		level: "module",
		type: "all",
		format: "smart",
		excludeOtp: false
	};

	// Options
	$scope.levels = [
		{value: "function",      text: "Function"},
		{value: "module",        text: "Module"},
		{value: "functionblock", text: "Set of modules"}
	];
	$scope.types = [
		{value: "all",           text: "Whole graph"},
		{value: "cycles",        text: "Cyclic sub-graph"}
	];
	$scope.formats = [
		{value: "text",          text: "Plain textual relations"},
		{value: "svg",           text: "Svg file"},
		{value: "smart",         text: "Smart graph"}
	];

	// Function level
	$scope.rootFunctionConfig = {
		query: autoComplete.createFunctionQuery(),
		allowClear: true,
		minimumInputLength: 1
	};
	$scope.excludeFunctionsConfig = {
		query: autoComplete.createFunctionQuery(),
		multiple: true,
		minimumInputLength: 1
	};
	$scope.noLeafFunctionsConfig = {
		query: autoComplete.createFunctionQuery(),
		multiple: true,
		minimumInputLength: 1
	};

	// Module level
	$scope.rootModuleConfig = {
		query: autoComplete.createModuleQuery(),
		allowClear: true,
		minimumInputLength: 1
	};
	$scope.excludeModulesConfig = {
		query: autoComplete.createModuleQuery(),
		multiple: true,
		minimumInputLength: 1
	};
	$scope.noLeafModulesConfig = {
		query: autoComplete.createModuleQuery(),
		multiple: true,
		minimumInputLength: 1
	};

	// Set of modules (functionblock) level

	var directories;
	var getDirectories = function() {
		if (directories) return $q.when(directories);
		return $http
			.get("api/getDirectories")
			.then(function(response) {
				return directories = _.map(response.data, function(dir) {
					return { id: dir, text: dir };
				});
			});
	};

	$scope.subjectsConfig = {
		multiple: true,
		query: function(query) {
			$scope.$apply(function() {
				getDirectories().then(function(directories) {
					var options = [];
					if (query.term.length) {
						options.push({
							id: query.term,
							text: query.term
						});
					}
					options.push.apply(options, directories);
					query.callback({
						results: options,
						more: false
					});
				});
			});
		}
	};

	// Visibilities
	$scope.typeVisible = function() {
		return (
			($scope.parameters.level != 'functionblock') &&
			($scope.parameters.format != 'text')
		);
	};
	$scope.rootFunctionVisible = function() {
		return (
			($scope.parameters.level == 'function')
		);
	};
	$scope.excludeFunctionsVisible = function() {
		return (
			($scope.parameters.level == 'function') &&
			($scope.parameters.format != 'text')
		);
	};
	$scope.rootModuleVisible = function() {
		return (
			($scope.parameters.level == 'module')
		);
	};
	$scope.excludeModulesVisible = function() {
		return (
			($scope.parameters.level == 'module') &&
			($scope.parameters.format != 'text')
		);
	};
	$scope.excludeOtpVisible = function() {
		return (
			($scope.parameters.level != 'functionblock') &&
			($scope.parameters.format != 'text')
		);
	};
	$scope.noLeafFunctionsVisible = function() {
		return (
			($scope.parameters.level == 'function') &&
			($scope.parameters.format != 'text')
		);
	};
	$scope.noLeafModulesVisible = function() {
		return (
			($scope.parameters.level == 'module') &&
			($scope.parameters.format != 'text')
		);
	};
	$scope.subjectsVisible = function() {
		return (
			($scope.parameters.level == 'functionblock')
		);
	};

	// Parameters binding
	$scope.$watch(graphs.parameters, function(parameters, old) {
		if (old === parameters) return; // don't clear with the initial empty parameters
		$scope.parameters = parameters;
	});

	// Results
	$scope.waiting = graphs.waiting;
	$scope.results = graphs.lastResults;

	$scope.generate = function() {
		if ($scope.parameters.format == "text" && $scope.parameters.level == "functionblock") {
			alert('Sorry, textual representation is not supported for "Set of modules" level.');
		} else {
			locationSearch.set("svg", false);
			graphs.execute($scope.parameters);
		}
	};

	$scope.seeSvg = function($event) {
		try {
			locationSearch.set("svg", true);
		} finally {
			$event.preventDefault();
		}
	};

	$scope.$watch(locationSearch.getterFor("svg"), function(svg) {
		$scope.displaySvg = svg;
	});
}
