'use strict';

angular.module('referl.services').factory('predefineds', function($http, queries, $rootScope) {

	var execute = function(node, query) {
		queries.execute({
			from: node,
			queryText: query.query
		});		
	};

	var queryFrom = function(node) {
		var parameters = queries.parameters();
		parameters.from = node;
		$rootScope.$broadcast("focusQueryInput");
	};

    var sendAsDynfun = function(node) {
        $http.post("api/saveAsDynfun", {
			ref: node.ref,
			display: node.display
		})
    };

	var predefineds = {
		get: function(file, index) {
			return $http
				.post("api/getPredefineds", {
					file: file,
					index: index + 1
				})
				.then(function(response) {
					return response.data;
				});
		},

		getAsContextMenu: function(file, index, addSeparator) {
			return predefineds.get(file, index).then(function(data) {
				var items = [];
				if (data.node) {
                    angular.forEach(data.queries, function(query) {
                        items.push({name: query.label, className: "wrap",
                            callback: function() {
                                if(query.query != "") execute(data.node, query);
                            }});
                    });
                    items.push({name: "Send \""+data.node.display+"\" as dynfun",
                        className: "wrap",
                        callback: function() {
                            sendAsDynfun(data.node);
                    }});
                    items.push({name: "Custom Query From Here", callback: function() {
                        queryFrom(data.node);
                    }});
                } else {
                    if(data.error) {
                        if(data.error=="nofile") {
                            items.push({name: "This file hasn't been added to the database",
                                className: "warning", disabled: true, callback: function() {
                                
                            }});
                        }
                        if(data.error=="nonode") {
                            items.push({name: "No entity at this position",
                                className: "warning", disabled: true, callback: function() {
                                
                            }});
                        }
                    }
                }
                    
                if (addSeparator) {
                    items.push("-----");
                }
        
                return items;
            });
        }
	};

	return predefineds;

});
